package com.ruoyi.web.controller.carrier.issued;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.carrier.TdCarrier;
import com.ruoyi.system.domain.carrier.TdCarrierIssued;
import com.ruoyi.web.controller.manager.CarrierIssuedManager;
import com.ruoyi.web.controller.manager.SysAreaManager;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * ClassName: CarrierIssuedController
 * Package: com.ruoyi.web.controller.carrier.Issued
 * Description:上级下发载体登记
 *
 * @Author wangxy
 * @Create 2024/9/24 10:23
 * @Version 1.0
 */
@Controller
@RequestMapping("/system/carrierIssued")
public class CarrierIssuedController extends BaseController {

    private String prefix = "system/carrierIssued";

    @Resource
    private CarrierIssuedManager carrierIssuedManager;


    @Resource
    private SysAreaManager sysAreaManager;


    @RequiresPermissions("system:carrierIssued:view")
    @GetMapping()
    public String carrierIssued() {
        return prefix + "/carrierIssued";
    }



    /**
     * 载体列表
     */
    @RequiresPermissions("system:carrierIssued:list")
    @PostMapping("/list")
    @ResponseBody
    public TableDataInfo list(TdCarrierIssued tdCarrierIssued) {
        startPage();
        List<TdCarrierIssued> carriers = carrierIssuedManager.selectTdCarrierList(tdCarrierIssued);
        return getDataTable(carriers);
    }


    /**
     * 导出文件下发列表
     */
    @RequiresPermissions("system:carrierIssued:export")
    @Log(title = "国家秘密载体", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    @ResponseBody
    public AjaxResult export(TdCarrierIssued tdCarrierIssued) {
        List<TdCarrierIssued> carriers = carrierIssuedManager.selectTdCarrierList(tdCarrierIssued);
        carriers.forEach(tdCarrier1 -> {
            tdCarrier1.setAreaCity(sysAreaManager.getAreaName(tdCarrier1.getAreaCity()));
            tdCarrier1.setAreaDistrict(sysAreaManager.getAreaName(tdCarrier1.getAreaDistrict()));
        });
        ExcelUtil<TdCarrierIssued> util = new ExcelUtil<>(TdCarrierIssued.class);
        return util.exportExcel(carriers, "国家秘密载体数据");
    }


    /**
     * 新增载体
     */
    @GetMapping("/add")
    public String add() {
        return prefix + "/add";
    }


    /**
     * 新增载体
     */
    @RequiresPermissions("system:carrierIssued:add")
    @Log(title = "国家秘密载体登记", businessType = BusinessType.INSERT)
    @PostMapping("/add")
    @ResponseBody
    public AjaxResult addSave(TdCarrierIssued tdCarrierIssued) {
        return toAjax(carrierIssuedManager.saveOrUpdate(tdCarrierIssued));
    }

    /**
     * 修改场国家秘密载体
     */
    @RequiresPermissions("system:carrierIssued:edit")
    @GetMapping("/edit/{id}")
    public String edit(@PathVariable("id") String id, ModelMap mmap) {
        TdCarrierIssued tdCarrierIssued = carrierIssuedManager.selectTdCarrier(id);
        mmap.put("tdCarrierIssued", tdCarrierIssued);
        return prefix + "/edit";
    }

    /**
     * 修改国家秘密载体
     */
    @RequiresPermissions("system:carrierIssued:edit")
    @Log(title = "国家秘密载体修改", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    @ResponseBody
    public AjaxResult editSave(TdCarrierIssued tdCarrierIssued) {
        return toAjax(carrierIssuedManager.saveOrUpdate(tdCarrierIssued));
    }

    /**
     * 详情国家秘密载体
     */
    @RequiresPermissions("system:carrierIssued:detail")
    @GetMapping("/detail/{id}")
    public String detail(@PathVariable("id") String id, ModelMap mmap) {
        TdCarrierIssued tdCarrierIssued = carrierIssuedManager.selectTdCarrier(id);
        mmap.put("tdCarrierIssued", tdCarrierIssued);
        return prefix + "/detail";
    }


    /**
     * 删除国家秘密载体
     */
    @RequiresPermissions("system:carrier:remove")
    @Log(title = "国家秘密载体", businessType = BusinessType.DELETE)
    @PostMapping("/remove")
    @ResponseBody
    public AjaxResult remove(String ids) {
        return toAjax(carrierIssuedManager.deletedTdCarrierByids(ids));
    }





}
