package com.ruoyi.web.controller.manager;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.text.CharSequenceUtil;
import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.utils.ShiroUtils;
import com.ruoyi.system.domain.check.TdCheckReport;
import com.ruoyi.system.domain.check.TdCheckType;
import com.ruoyi.system.domain.check.dto.*;
import com.ruoyi.system.service.ISysDictDataService;
import com.ruoyi.system.service.check.TdCheckReportService;
import com.ruoyi.system.service.check.TdCheckTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * packageName com.ruoyi.web.controller.manager
 *
 * @author wangxy
 * @version JDK 8
 * @className CheckReportManager
 * @date 2024/8/15
 * @description 检查报告
 */
@Component
public class CheckReportManager {


    @Resource
    private TdCheckReportService checkReportService;

    @Resource
    private TdCheckTypeService checkTypeService;


    @Autowired
    private ISysDictDataService dictDataService;



    public List<TdCheckReport> selectTdCheckReportList(CheckReportDTO tdCheckReport) {
        return checkReportService.selectTdCheckReportList(tdCheckReport);
    }


    public List<TdCheckReport> selectTdCheckResultList(TdCheckReport tdCheckReport) {
        return checkReportService.selectTdCheckResultList(tdCheckReport);
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean saveOrUpdate(TdCheckReportDTO checkReportDTO) {
        TdCheckReport tdCheckReport = Convert.convert(TdCheckReport.class, checkReportDTO);
        if (CharSequenceUtil.isNotBlank(checkReportDTO.getCheckId())) {
            checkTypeService.lambdaUpdate()
                    .eq(TdCheckType::getCheckId, checkReportDTO.getCheckId())
                    .remove();
            tdCheckReport.setUpdateTime(new Date());
            tdCheckReport.setUpdateBy(ShiroUtils.getSysUser().getUserName());
        } else {
            tdCheckReport.setDepart(ShiroUtils.getSysUser().getDept().getDeptName());
            tdCheckReport.setAdduser(ShiroUtils.getSysUser().getUserName());
            tdCheckReport.setCreateTime(new Date());
            tdCheckReport.setCreateBy(ShiroUtils.getSysUser().getUserName());
        }
        tdCheckReport.setDeptId(ShiroUtils.getSysUser().getDeptId());
        checkReportService.saveOrUpdate(tdCheckReport);
        List<TdCheckType> list = Convert.toList(TdCheckType.class, checkReportDTO.getCheckTypeDTOS());
        list.forEach(checkReport -> checkReport.setCheckId(tdCheckReport.getCheckId()));
        return checkTypeService.saveBatch(list);
    }

    public List<CheckTypeItemDTO> getCheckTypeDTO(String checkType,String typeName) {
        List<CheckTypeItemDTO> checkTypeItemDTOList =new ArrayList<>();
        List<SysDictData> dataList = dictDataService.selectDictDataByType(checkType);
        dataList.forEach(sysDictData2 -> checkTypeItemDTOList.add(CheckTypeItemDTO.builder()
                .typeName(typeName)
                .checkType(checkType)
                .checkItems(sysDictData2.getDictLabel())
                .itemsType(sysDictData2.getDictCode())
                .typeContent(sysDictData2.getDictValue())
                .score(sysDictData2.getCssClass())
                .deductionCriteria(sysDictData2.getRemark())
                .build()));

        return checkTypeItemDTOList;
    }

    public List<CheckTypeDTO> getCheckTypeDtoParent(String type) {
        List<CheckTypeDTO> checkTypeDtoS = new ArrayList<>();
        List<SysDictData> list = dictDataService.selectDictDataByType(type);
        list.forEach(sysDictData1 -> checkTypeDtoS.add(CheckTypeDTO.builder()
                .typeName(sysDictData1.getDictLabel())
                .checkType(sysDictData1.getDictValue()).build()));
        return checkTypeDtoS;
    }


    @Transactional(rollbackFor = Exception.class)
    public boolean deletedTdCheckReportIds(String ids) {
        List<String> list = Arrays.asList(Convert.toStrArray(ids));
        checkTypeService.lambdaUpdate().in(TdCheckType::getCheckId,list).remove();
        return checkReportService.removeByIds(list);
    }


    public TdCheckReportDTO getTdCheckReportDTO(String id) {
        TdCheckReport tdCheckReport = checkReportService.getById(id);
        TdCheckReportDTO tdCheckReportDto = Convert.convert(TdCheckReportDTO.class, tdCheckReport);
        List<TdCheckType> list = checkTypeService.lambdaQuery().eq(TdCheckType::getCheckId, id).list();
        if(CollUtil.isNotEmpty(list)){
            List<TdCheckTypeDTO> tdCheckTypeDtoS = Convert.toList(TdCheckTypeDTO.class, list);
            tdCheckReportDto.setCheckTypeDTOS(tdCheckTypeDtoS);
        }
        return tdCheckReportDto;

    }


}
