package com.ruoyi.web.controller.secret;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.secreting.TdSecreting;
import com.ruoyi.web.controller.manager.SecretManager;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author Administrator
 */
@Controller
@RequestMapping("/system/secreting")
public class TdSecretingController extends BaseController {
    private String prefix = "system/secreting";

    @Resource
    private SecretManager secretManager;

    @RequiresPermissions("system:secreting:view")
    @GetMapping()
    public String secreting() {
        return prefix + "/secreting";
    }

    /**
     * 定密列表
     */
    @RequiresPermissions("system:secreting:list")
    @PostMapping("/list")
    @ResponseBody
    public TableDataInfo list(TdSecreting tdSecreting) {
        startPage();
        List<TdSecreting> tdSecretings = secretManager.selectSecretList(tdSecreting);
        return getDataTable(tdSecretings);
    }

    /**
     * 新增定密事项
     */
    @GetMapping("/add")
    public String add() {
        return prefix + "/add";
    }

    /**
     * 新增定密事项
     */
    @RequiresPermissions("system:secreting:add")
    @Log(title = "定密事项管理", businessType = BusinessType.INSERT)
    @PostMapping("/add")
    @ResponseBody
    public AjaxResult addSave(TdSecreting tdSecreting) {
        return toAjax(secretManager.saveOrUpdate(tdSecreting));
    }

    /**
     * 修改定密事项
     */
    @RequiresPermissions("system:secreting:edit")
    @GetMapping("/edit/{id}")
    public String edit(@PathVariable("id") String id, ModelMap mmap) {
        TdSecreting tdSecreting = secretManager.selectSecreting(id);
        mmap.put("tdSecreting", tdSecreting);
        return prefix + "/edit";
    }

    /**
     * 修改定密事项
     */
    @RequiresPermissions("system:secreting:edit")
    @Log(title = "定密事项管理", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    @ResponseBody
    public AjaxResult editSave(TdSecreting tdSecreting) {
        return toAjax(secretManager.saveOrUpdate(tdSecreting));
    }

    /**
     * 定密事项详情
     */
    @RequiresPermissions("system:secreting:detail")
    @GetMapping("/detail/{id}")
    public String detail(@PathVariable("id") String id, ModelMap mmap) {
        TdSecreting tdSecreting = secretManager.selectSecreting(id);
        mmap.put("tdSecreting", tdSecreting);
        return prefix + "/detail";
    }

    /**
     * 删除定密事项
     */
    @RequiresPermissions("system:secreting:remove")
    @Log(title = "定密事项管理", businessType = BusinessType.DELETE)
    @PostMapping("/remove")
    @ResponseBody
    public AjaxResult remove(String ids) {
        return toAjax(secretManager.deleteSecretingByIds(ids));
    }

}
