package com.ruoyi.web.controller.system.count;

import cn.hutool.core.text.StrPool;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.ObtainLastSixMonthsUtil;
import com.ruoyi.system.domain.TdMeeting;
import com.ruoyi.system.service.TdMeetingService;
import com.ruoyi.web.controller.manager.TdMeetingManager;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicReference;

/**
 * 涉密会议统计
 */
@Controller
@RequestMapping("/system/meetingnum")
public class SysMeetingnumController extends BaseController {
    
    private String prefix = "system/tdmeeting/meetingnum";
    @Resource
    private TdMeetingService tdMeetingService;
    @Resource
    private TdMeetingManager tdMeetingManager;

    @RequiresPermissions("system:meetingnum:view")
    @GetMapping()
    public String meetingnum() {
        return prefix + "/meetingnum";
    }

    /**
     * 会议列表
     */
    @PostMapping("/list")
    @ResponseBody
    public TableDataInfo list(TdMeeting tdMeeting) {
        startPage();
        List<TdMeeting> tdMeetings = tdMeetingManager.selectTdMeetingList(tdMeeting);
        return getDataTable(tdMeetings);
    }

    /**
     * 会议12个月统计
     *
     * @return
     */
    @PostMapping("/countMeetingMonth")
    @ResponseBody
    public AjaxResult countMeetingMonth() {
        List<String> recentlyTwelveMonth = ObtainLastSixMonthsUtil.getRecentlySixMonth();
        List<Long> yAxisData = new ArrayList<>();
        List<Long> yAxisDataW = new ArrayList<>();
        List<Long> yAxisDataXs= new ArrayList<>();
        List<Long> yAxisDataXx = new ArrayList<>();
        recentlyTwelveMonth.forEach(month -> {
            // 获取指定月份的最大日期
            String lastDayOfMonth = ObtainLastSixMonthsUtil.getLastDayOfMonth(month);
            // 获取指定天的最大时间
            Date date = Date.from(LocalDate.parse(lastDayOfMonth).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date maxDayTime = ObtainLastSixMonthsUtil.getEndOfDay(date);
            // 获取最小时间
            date = Date.from(LocalDate.parse(month.concat(StrPool.DASHED).concat("01")).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date minDayTime = ObtainLastSixMonthsUtil.getStartOfDay(date);
            AtomicReference<Long> yAxis = new AtomicReference<>(0L);
            Integer count = new LambdaQueryChainWrapper<>(tdMeetingService.getBaseMapper())
                    .ge(TdMeeting::getMeetingTime, minDayTime)
                    .le(TdMeeting::getMeetingTime, maxDayTime).eq(TdMeeting::getMeetingState,1).count();
            if (Objects.isNull(count)) {
                count = 0;
            }
            yAxis.set(Long.valueOf(count));
            yAxisData.add(yAxis.get());
            AtomicReference<Long> yAxisW = new AtomicReference<>(0L);
            Integer countW = new LambdaQueryChainWrapper<>(tdMeetingService.getBaseMapper())
                    .ge(TdMeeting::getMeetingTime, minDayTime)
                    .le(TdMeeting::getMeetingTime, maxDayTime).eq(TdMeeting::getMeetingState,0).count();
            if (Objects.isNull(countW)) {
                countW = 0;
            }
            yAxisW.set(Long.valueOf(countW));
            yAxisDataW.add(yAxisW.get());

            AtomicReference<Long> yAxisXs = new AtomicReference<>(0L);
            Integer countXs= new LambdaQueryChainWrapper<>(tdMeetingService.getBaseMapper())
                    .ge(TdMeeting::getMeetingTime, minDayTime)
                    .le(TdMeeting::getMeetingTime, maxDayTime).eq(TdMeeting::getMeetingType,0).count();
            if (Objects.isNull(countXs)) {
                countXs = 0;
            }
            yAxisXs.set(Long.valueOf(countXs));
            yAxisDataXs.add(yAxisXs.get());

            AtomicReference<Long> yAxisXx = new AtomicReference<>(0L);
            Integer countXx= new LambdaQueryChainWrapper<>(tdMeetingService.getBaseMapper())
                    .ge(TdMeeting::getMeetingTime, minDayTime)
                    .le(TdMeeting::getMeetingTime, maxDayTime).eq(TdMeeting::getMeetingType,1).count();
            if (Objects.isNull(countXx)) {
                countXx = 0;
            }
            yAxisXx.set(Long.valueOf(countXx));
            yAxisDataXx.add(yAxisXx.get());

        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", recentlyTwelveMonth.toArray());
        obj.put("yAxisData", yAxisData);
        obj.put("yAxisDataW", yAxisDataW);
        obj.put("yAxisDataXs", yAxisDataXs);
        obj.put("yAxisDataXx", yAxisDataXx);
        return AjaxResult.success(obj);
    }


    /**
     * 会议详情
     */
    @GetMapping("/detail/{id}")
    public String view(@PathVariable("id") String id, ModelMap mmap) {
        TdMeeting meeting = tdMeetingManager.selectTdMeeting(id);
        mmap.put("meeting", meeting);
        return prefix + "/detail";
    }
}