package com.ruoyi.web.controller.system.count;

import cn.hutool.core.text.StrPool;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.ObtainLastSixMonthsUtil;
import com.ruoyi.system.domain.TdPropertyEment;
import com.ruoyi.system.domain.TdPropertyInfo;
import com.ruoyi.system.domain.TdPropertyManager;
import com.ruoyi.system.domain.check.TdCheckReport;
import com.ruoyi.system.domain.count.*;
import com.ruoyi.system.service.ISysDictDataService;
import com.ruoyi.system.service.ITdPropertyInfoService;
import com.ruoyi.system.service.ITdPropertyManagerService;
import com.ruoyi.system.service.TdPropertyEmentService;
import lombok.val;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

@Controller
@RequestMapping("/system/propertynum")
public class SysPropertynumController extends BaseController {
    private String prefix = "system/property/propertynum";
    @Autowired
    private ITdPropertyManagerService tdPropertyManagerService;
    @Autowired
    private ITdPropertyInfoService tdPropertyInfoService;

    @Resource
    private TdPropertyEmentService tdPropertyEmentService;

    @RequiresPermissions("system:propertynum:view")
    @GetMapping()
    public String property() {
        return prefix + "/propertynum";
    }

    /**
     * 查询密品登记列表
     */
    @RequiresPermissions("system:propertynum:list")
    @PostMapping("/list")
    @ResponseBody
    public TableDataInfo list(TdPropertyManager tdPropertyManager) {
        startPage();
        List<TdPropertyManager> list = tdPropertyManagerService.selectTdPropertyManagerList(tdPropertyManager);
        return getDataTable(list);
    }

    /**
     * 打印界面
     */
    @RequiresPermissions("system:propertynum:print")
    @GetMapping("/print/{useId}")
    @Log(title = "涉密信息设备统计", businessType = BusinessType.PRINT)
    public String print(@PathVariable("useId") String useId, ModelMap mmap) {
        List<TdPropertyInfo> tdPropertyInfos = tdPropertyInfoService.selectTdPropertyInfoByUseId(useId);
        TdPropertyManager tdPropertyManager = tdPropertyManagerService.selectTdPropertyManagerByUseId(useId);
        mmap.put("tdPropertyInfos", tdPropertyInfos);
        mmap.put("tdPropertyManager", tdPropertyManager);
        mmap.put("sysuser", getSysUser());
        return prefix + "/print";
    }

    /**
     * 打印界面
     */
    @RequiresPermissions("system:propertynum:print")
    @GetMapping("/printEment/{useId}")
    @Log(title = "涉密设备统计", businessType = BusinessType.PRINT)
    public String printEment(@PathVariable("useId") String useId, ModelMap mmap) {
        TdPropertyManager tdPropertyManager = tdPropertyManagerService.selectTdPropertyManagerByUseId(useId);
        List<TdPropertyEment> tdPropertyEments = tdPropertyEmentService.lambdaQuery()
                .eq(TdPropertyEment::getUseId, useId)
                .orderByDesc(TdPropertyEment::getCreateTime)
                .list();
        mmap.put("tdPropertyEments", tdPropertyEments);
        mmap.put("tdPropertyManager", tdPropertyManager);
        mmap.put("sysuser", getSysUser());
        return prefix + "/printEment";
    }


    @Autowired
    private ISysDictDataService dictDataService;

    /**
     * 密品统计
     *
     * @return com.ruoyi.common.core.domain.AjaxResult
     */

    @PostMapping("/countInfo")
    @ResponseBody
    public AjaxResult countInfo() {
        SysDictData sysDictData = new SysDictData();
        sysDictData.setDictType("sys_sm_property");
        List<SysDictData> smType = dictDataService.selectDictDataList(sysDictData);
        List<String> list = smType.stream().map(SysDictData::getDictLabel).collect(Collectors.toList());
        List<Long> yAxisData = new ArrayList<>();
        smType.forEach(xAxis -> {
            AtomicReference<Long> yAxis = new AtomicReference<>(0L);
            Integer count = tdPropertyInfoService.lambdaQuery()
                    .eq(TdPropertyInfo::getPropertyType, xAxis.getDictValue())
                    .count();
            yAxis.set(Long.valueOf(count));

            yAxisData.add(yAxis.get());
        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", list.toArray());
        obj.put("yAxisData", yAxisData);
        return AjaxResult.success(obj);
    }

    private List<String> bf = Arrays.asList("信息设备未报废", "信息设备已报废","设备未报废", "设备已报废");

    private List<String> xh = Arrays.asList("未销毁", "已销毁");

    /**
     * 统计密品报废统计
     *
     * @return com.ruoyi.common.core.domain.AjaxResult
     */
    @PostMapping("/countBF")
    @ResponseBody
    public AjaxResult countbf() {
        PropertyStateCountDTO propertyStateCountDTO = tdPropertyInfoService.selectCountBF();
        List<JSONObject> datas = bf.stream()
                .map(name -> {
                    AtomicReference<Object> value = new AtomicReference<>(0L);
                    if ("信息设备未报废".equals(name)) {
                        value.set(propertyStateCountDTO.getWbf());
                    } else if("信息设备已报废".equals(name)) {
                        value.set(propertyStateCountDTO.getYbf());
                    } else if("设备未报废".equals(name)) {
                        value.set(tdPropertyEmentService.lambdaQuery()
                                .eq(TdPropertyEment::getDestoryType,0)
                                .count());
                    }else{
                        value.set(tdPropertyEmentService.lambdaQuery()
                                .eq(TdPropertyEment::getDestoryType,1)
                                .count());
                    }
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("value", value.get());
                    jsonObject.put("name", name);
                    return jsonObject;
                })
                .collect(Collectors.toList());
        return AjaxResult.success(datas);
    }

    /**
     * 统计密品销毁统计
     *
     * @return com.ruoyi.common.core.domain.AjaxResult
     */
    @PostMapping("/countXH")
    @ResponseBody
    public AjaxResult countXH() {
        PropertyDesCountDTO propertyDesCountDTO = tdPropertyInfoService.selectCountXH();
        List<JSONObject> datas = xh.stream()
                .map(name -> {
                    AtomicReference<Object> value = new AtomicReference<>(0L);
                    if ("未销毁".equals(name)) {
                        value.set(propertyDesCountDTO.getWxh());
                    } else {
                        value.set(propertyDesCountDTO.getYxh());
                    }
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("value", value.get());
                    jsonObject.put("name", name);
                    return jsonObject;
                })
                .collect(Collectors.toList());
        return AjaxResult.success(datas);
    }

    /**
     *
     *近6个月统计涉密信息设备
     * @return
     */
    @PostMapping("/countMessageMonth")
    @ResponseBody
    public AjaxResult countMessageMonth() {
        List<String> recentlySixMonth = ObtainLastSixMonthsUtil.getRecentlySixMonth();
        List<Long> yAxisData = new ArrayList<>();
        List<Long> yAxisDataSb = new ArrayList<>();
        recentlySixMonth.forEach(month -> {
            // 获取指定月份的最大日期
            String lastDayOfMonth = ObtainLastSixMonthsUtil.getLastDayOfMonth(month);
            // 获取指定天的最大时间
            Date date = Date.from(LocalDate.parse(lastDayOfMonth).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date maxDayTime = ObtainLastSixMonthsUtil.getEndOfDay(date);
            // 获取最小时间
            date = Date.from(LocalDate.parse(month.concat(StrPool.DASHED).concat("01")).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date minDayTime = ObtainLastSixMonthsUtil.getStartOfDay(date);
            AtomicReference<Long> yAxis = new AtomicReference<>(0L);
            PropertyManagerDTO tdPropertyManager = new PropertyManagerDTO();
            tdPropertyManager.setMinDayTime(minDayTime);
            tdPropertyManager.setMaxDayTime(maxDayTime);
            tdPropertyManager.setSecretsType("1");
            Integer count = tdPropertyManagerService.countMessageMonth(tdPropertyManager);
            if(Objects.isNull(count)){
                count = 0;
            }
            yAxis.set(Long.valueOf(count));
            yAxisData.add(yAxis.get());
            AtomicReference<Long> yAxisSb = new AtomicReference<>(0L);
            PropertyManagerDTO tdPropertManager = new PropertyManagerDTO();
            tdPropertManager.setMinDayTime(minDayTime);
            tdPropertManager.setMaxDayTime(maxDayTime);
            tdPropertManager.setSecretsType("2");
            Integer countSecrets = tdPropertyManagerService.countSecretsMonth(tdPropertManager);
            if(Objects.isNull(countSecrets)){
                countSecrets = 0;
            }
            yAxisSb.set(Long.valueOf(countSecrets));
            yAxisDataSb.add(yAxisSb.get());
        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", recentlySixMonth.toArray());
        //涉密信息设备
        obj.put("yAxisData", yAxisData);
        //涉密设备
        obj.put("yAxisDataSb", yAxisDataSb);
        return AjaxResult.success(obj);
    }

    /**
     *
     *检查近6个月统计涉密设备
     * @return
     */
    @PostMapping("/countSecretsMonth")
    @ResponseBody
    public AjaxResult countSecretsMonth() {
        List<String> recentlySixMonth = ObtainLastSixMonthsUtil.getRecentlySixMonth();
        List<Long> yAxisData = new ArrayList<>();
        List<Long> yAxisDataWx = new ArrayList<>();
        List<Long> yAxisDataSb = new ArrayList<>();
        List<Long> yAxisDataWxSb = new ArrayList<>();
        recentlySixMonth.forEach(month -> {
            // 获取指定月份的最大日期
            String lastDayOfMonth = ObtainLastSixMonthsUtil.getLastDayOfMonth(month);
            // 获取指定天的最大时间
            Date date = Date.from(LocalDate.parse(lastDayOfMonth).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date maxDayTime = ObtainLastSixMonthsUtil.getEndOfDay(date);
            // 获取最小时间
            date = Date.from(LocalDate.parse(month.concat(StrPool.DASHED).concat("01")).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date minDayTime = ObtainLastSixMonthsUtil.getStartOfDay(date);
            AtomicReference<Long> yAxis = new AtomicReference<>(0L);
            PropertyManagerDTO tdPropertyManager = new PropertyManagerDTO();
            tdPropertyManager.setMinDayTime(minDayTime);
            tdPropertyManager.setMaxDayTime(maxDayTime);
            tdPropertyManager.setSecretsType("1");
            tdPropertyManager.setDestoryState("1");
            Integer count = tdPropertyManagerService.countDestoryMonth(tdPropertyManager);
            if(Objects.isNull(count)){
                count = 0;
            }
            yAxis.set(Long.valueOf(count));
            yAxisData.add(yAxis.get());

            AtomicReference<Long> yAxisWx = new AtomicReference<>(0L);
            PropertyManagerDTO tdPropertManager = new PropertyManagerDTO();
            tdPropertManager.setMinDayTime(minDayTime);
            tdPropertManager.setMaxDayTime(maxDayTime);
            tdPropertManager.setSecretsType("1");
            Integer countWx = tdPropertyManagerService.countMessageWxMonth(tdPropertManager);
            if(Objects.isNull(countWx)){
                countWx = 0;
            }
            yAxisWx.set(Long.valueOf(countWx));
            yAxisDataWx.add(yAxisWx.get());


            AtomicReference<Long> yAxisSb = new AtomicReference<>(0L);
            PropertyManagerDTO tdProperty = new PropertyManagerDTO();
            tdProperty.setMinDayTime(minDayTime);
            tdProperty.setMaxDayTime(maxDayTime);
            tdProperty.setSecretsType("2");
            tdProperty.setDestoryState("1");
            Integer countSb = tdPropertyManagerService.countSecretsDestoryMonth(tdProperty);
            if(Objects.isNull(countSb)){
                countSb = 0;
            }
            yAxisSb.set(Long.valueOf(countSb));
            yAxisDataSb.add(yAxisSb.get());

            AtomicReference<Long> yAxisSbWx = new AtomicReference<>(0L);
            PropertyManagerDTO tdPropertWx = new PropertyManagerDTO();
            tdPropertWx.setMinDayTime(minDayTime);
            tdPropertWx.setMaxDayTime(maxDayTime);
            tdPropertWx.setSecretsType("2");
            Integer countSbWx = tdPropertyManagerService.countSecretsWxMonth(tdPropertWx);
            if(Objects.isNull(countSbWx)){
                countSbWx = 0;
            }
            yAxisSbWx.set(Long.valueOf(countSbWx));
            yAxisDataWxSb.add(yAxisSbWx.get());

        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", recentlySixMonth.toArray());
        //已销毁
        obj.put("yAxisData", yAxisData);
        //未销毁
        obj.put("yAxisDataWx", yAxisDataWx);
        //设备销毁
        obj.put("yAxisDataSb", yAxisDataSb);
        //设备未销毁
        obj.put("yAxisDataWxSb", yAxisDataWxSb);
        return AjaxResult.success(obj);
    }

}
