package com.ruoyi.web.controller.system.count;

import cn.hutool.core.text.StrPool;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.ObtainLastSixMonthsUtil;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.TdTrain;
import com.ruoyi.system.domain.check.TdCheckReport;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.service.ITdTrainService;
import com.ruoyi.web.controller.manager.SysAreaManager;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicReference;

@Controller
@RequestMapping("/system/trainnum")
public class SysTrainnumController extends BaseController {
    private String prefix = "system/trainnum";
    @Autowired
    private ITdTrainService tdTrainService;

    @Resource
    private SysAreaManager sysAreaManager;


    @Resource
    private ISysUserService sysUserService;

    @RequiresPermissions("system:trainnum:view")
    @GetMapping()
    public String trainnum()
    {
        return prefix + "/trainnum";
    }

    /**
     * 培训列表
     * @param tdTrain
     * @return
     */
    @RequiresPermissions("system:trainnum:list")
    @PostMapping("/list")
    @ResponseBody
    public TableDataInfo list(TdTrain tdTrain)
    {
        startPage();
        List<TdTrain> list = tdTrainService.selectTdTrainList(tdTrain);
        return getDataTable(list);
    }

    /**
     * 导出培训数据
     * @param tdTrain
     * @return
     */
    @Log(title = "培训统计", businessType = BusinessType.EXPORT)
    @RequiresPermissions("system:trainnum:export")
    @PostMapping("/export")
    @ResponseBody
    public AjaxResult export(TdTrain tdTrain)
    {
        List<TdTrain> list = tdTrainService.selectTdTrainList(tdTrain);
        list.forEach(tdTrain1 -> {
            tdTrain1.setAREAID(sysAreaManager.getAreaName(tdTrain1.getAREAID()));
            tdTrain1.setFRAMEWORK(sysAreaManager.getAreaName(tdTrain1.getFRAMEWORK()));
             SysUser sysUser = sysUserService.selectUserById(Long.valueOf(tdTrain1.getUSERNAME()));
             if(Objects.nonNull(sysUser)){
                 tdTrain1.setUSERNAME(sysUser.getUserName());
             }
        });
        ExcelUtil<TdTrain> util = new ExcelUtil<TdTrain>(TdTrain.class);
        return util.exportExcel(list, "培训统计数据");
    }

    /**
     * 详细
     */
    @RequiresPermissions("system:trainnum:detail")
    @GetMapping("/detail/{ID}")
    public String detail(@PathVariable("ID") Long ID, ModelMap mmap)
    {
        mmap.put("tdTrain", tdTrainService.selectTdTrainByID(ID));
        SysUser user = getSysUser();
        mmap.put("user", user);
        return prefix + "/detail";
    }

    @ApiOperation("培训统计数据")
    @PostMapping("/countList")
    @ResponseBody
    public AjaxResult countList() {
        return AjaxResult.success(tdTrainService.selectCountList());
    }

    /**
     *
     *检查近6个月审核通过统计
     * @return
     */
    @PostMapping("/countTrainMonth")
    @ResponseBody
    public AjaxResult countTrainMonth() {
        List<String> recentlySixMonth = ObtainLastSixMonthsUtil.getRecentlySixMonth();
        List<Long> yAxisData = new ArrayList<>();
        recentlySixMonth.forEach(month -> {
            // 获取指定月份的最大日期
            String lastDayOfMonth = ObtainLastSixMonthsUtil.getLastDayOfMonth(month);
            // 获取指定天的最大时间
            Date date = Date.from(LocalDate.parse(lastDayOfMonth).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date maxDayTime = ObtainLastSixMonthsUtil.getEndOfDay(date);
            // 获取最小时间
            date = Date.from(LocalDate.parse(month.concat(StrPool.DASHED).concat("01")).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date minDayTime = ObtainLastSixMonthsUtil.getStartOfDay(date);
            AtomicReference<Long> yAxis = new AtomicReference<>(0L);
            Integer count = new LambdaQueryChainWrapper<>(tdTrainService.getBaseMapper())
                    .ge(TdTrain::getCreateDate, minDayTime)
                    .le(TdTrain::getCreateDate, maxDayTime)
                    .eq(TdTrain::getTrainState,0)
                    .count();
            if(Objects.isNull(count)){
                count = 0;
            }
            yAxis.set(Long.valueOf(count));
            yAxisData.add(yAxis.get());
        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", recentlySixMonth.toArray());
        obj.put("yAxisData", yAxisData);
        return AjaxResult.success(obj);
    }

    /**
     *
     *检查近6个月待审核统计
     * @return
     */
    @PostMapping("/countTrainNoMonth")
    @ResponseBody
    public AjaxResult countTrainNoMonth() {
        List<String> recentlySixMonth = ObtainLastSixMonthsUtil.getRecentlySixMonth();
        List<Long> yAxisData = new ArrayList<>();
        recentlySixMonth.forEach(month -> {
            // 获取指定月份的最大日期
            String lastDayOfMonth = ObtainLastSixMonthsUtil.getLastDayOfMonth(month);
            // 获取指定天的最大时间
            Date date = Date.from(LocalDate.parse(lastDayOfMonth).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date maxDayTime = ObtainLastSixMonthsUtil.getEndOfDay(date);
            // 获取最小时间
            date = Date.from(LocalDate.parse(month.concat(StrPool.DASHED).concat("01")).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date minDayTime = ObtainLastSixMonthsUtil.getStartOfDay(date);
            AtomicReference<Long> yAxis = new AtomicReference<>(0L);
            Integer count = new LambdaQueryChainWrapper<>(tdTrainService.getBaseMapper())
                    .ge(TdTrain::getCreateDate, minDayTime)
                    .le(TdTrain::getCreateDate, maxDayTime)
                    .eq(TdTrain::getTrainState,2)
                    .count();
            if(Objects.isNull(count)){
                count = 0;
            }
            yAxis.set(Long.valueOf(count));
            yAxisData.add(yAxis.get());
        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", recentlySixMonth.toArray());
        obj.put("yAxisData", yAxisData);
        return AjaxResult.success(obj);
    }

}
