package com.ruoyi.web.controller.system.count;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.text.StrPool;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysPostVO;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.ObtainLastSixMonthsUtil;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.SysPost;
import com.ruoyi.system.domain.SysUserPost;
import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.domain.check.TdCheckReport;
import com.ruoyi.system.domain.count.UserCountDTO;
import com.ruoyi.system.domain.count.UserSmCountDTO;
import com.ruoyi.system.service.*;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * 涉密人员统计
 */
@Controller
@RequestMapping("/system/usernum")
public class SysUsernumController extends BaseController {
    private String prefix = "system/usernum";

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysPostService postService;
    @Autowired
    private ISysRoleService roleService;
    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private SysUserPostService sysUserPostService;

    @RequiresPermissions("system:usernum:view")
    @GetMapping()
    public String user() {
        return prefix + "/usernum";
    }

    /**
     * 用户列表
     *
     * @param user
     * @return
     */
    @RequiresPermissions("system:usernum:list")
    @PostMapping("/list")
    @ResponseBody
    public TableDataInfo list(SysUser user, SysDept dept) {
        startPage();
        List<SysUser> sysuser = new ArrayList<>();
        List<SysUser> list = userService.selectUserGroupBydept(user);
        List<SysDept> sysDepts = deptService.selectDeptList(dept);
        for (SysDept sysDept : sysDepts) {
            for (SysUser user1 : list) {
                if (sysDept.getDeptName().equals(user1.getDept().getDeptName())) {
                    sysuser.add(user1);
                }
            }
        }
        return getDataTable(sysuser);
    }

    /**
     * 导出用户
     *
     * @param user
     * @return
     */
    @Log(title = "用户统计", businessType = BusinessType.EXPORT)
    @RequiresPermissions("system:usernum:export")
    @PostMapping("/export")
    @ResponseBody
    public AjaxResult export(SysUser user) {
        List<SysUser> list = userService.selectUserList(user);
        ExcelUtil<SysUser> util = new ExcelUtil<SysUser>(SysUser.class);
        return util.exportExcel(list, "用户统计数据");
    }

    /**
     * 打印
     */
    @RequiresPermissions("system:usernum:detail")
    @GetMapping("/print/{deptId}")
    public String print(@PathVariable("deptId") Long deptId, SysUser user, ModelMap mmap) {
        List<SysUser> list = userService.lambdaQuery().eq(SysUser::getDeptId, deptId)
                .eq(SysUser::getDelFlag, 0)
                .ne(SysUser::getUserId, 1)
                .list();
        list.forEach(sysuser -> {
            List<SysUserPost> postList = sysUserPostService.lambdaQuery().
                    eq(SysUserPost::getUserId, sysuser.getUserId()).list();
            List<Long> postIds = postList.stream().map(SysUserPost::getPostId).collect(Collectors.toList());
            if (CollUtil.isNotEmpty(postIds)) {
                List<SysPost> sysPostList = postService.lambdaQuery().in(SysPost::getPostId, postIds).list();
                sysuser.setPosts(Convert.toList(SysPostVO.class, sysPostList));
            }
        });
        mmap.put("list", list);
        mmap.put("dept", deptService.selectDeptById(deptId));
        mmap.put("sysuser", getSysUser());
        return prefix + "/print";
    }


    private List<String> sharType = Arrays.asList("已登记", "已申报", "已审核", "已离职","已离岗");


    private List<String> smType = Arrays.asList("一般涉密人员", "重要涉密人员", "核心涉密人员");


    @PostMapping("/countSm")
    @ResponseBody
    public AjaxResult countSm() {
        UserSmCountDTO userSmCountDTO = userService.selectCountSmUser();
        List<Long> yAxisData = new ArrayList<>();
        smType.forEach(xAxis -> {
            AtomicReference<Long> yAxis = new AtomicReference<>(0L);
            if("一般涉密人员".equals(xAxis)){
                yAxis.set(Long.valueOf(userSmCountDTO.getYb()));
            }else if("重要涉密人员".equals(xAxis)){
                yAxis.set(Long.valueOf(userSmCountDTO.getZy()));
            }else{
                yAxis.set(Long.valueOf(userSmCountDTO.getHx()));
            }
            yAxisData.add(yAxis.get());
        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", smType.toArray());
        obj.put("yAxisData", yAxisData);
        return AjaxResult.success(obj);
    }

    /**
     * 
     *  统计人员已登记、已申报、已审核、已离职
     * @return com.ruoyi.common.core.domain.AjaxResult
     */
    @PostMapping("/countUser")
    @ResponseBody
    public AjaxResult countUser() {
        UserCountDTO userCountDTO = userService.selectCountUser();
        List<JSONObject> datas = sharType.stream()
                .map(name -> {
                    AtomicReference<Object> value = new AtomicReference<>(0L);
                    if("已登记".equals(name)){
                        value.set(userCountDTO.getYdj());
                    }else if("已申报".equals(name)){
                        value.set(userCountDTO.getYsb());
                    }else if("已审核".equals(name)){
                        value.set(userCountDTO.getYsh());
                    }else if("已离职".equals(name)){
                        value.set(userCountDTO.getYlz());
                    }else{
                        value.set(userCountDTO.getYlg());
                    }
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("value", value.get());
                    jsonObject.put("name", name);
                    return jsonObject;
                })
                .collect(Collectors.toList());
        return AjaxResult.success(datas);
    }


    /**
     *
     *近6个月已登记统计
     * @return
     */
    @PostMapping("/countCheckMonth")
    @ResponseBody
    public AjaxResult countCheckMonth() {
        List<String> recentlySixMonth = ObtainLastSixMonthsUtil.getRecentlySixMonth();
        List<Long> yAxisData = new ArrayList<>();
        recentlySixMonth.forEach(month -> {
            // 获取指定月份的最大日期
            String lastDayOfMonth = ObtainLastSixMonthsUtil.getLastDayOfMonth(month);
            // 获取指定天的最大时间
            Date date = Date.from(LocalDate.parse(lastDayOfMonth).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date maxDayTime = ObtainLastSixMonthsUtil.getEndOfDay(date);
            // 获取最小时间
            date = Date.from(LocalDate.parse(month.concat(StrPool.DASHED).concat("01")).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date minDayTime = ObtainLastSixMonthsUtil.getStartOfDay(date);
            AtomicReference<Long> yAxis = new AtomicReference<>(0L);
            Integer count = new LambdaQueryChainWrapper<>(userService.getBaseMapper())
                    .ge(SysUser::getCreateTime, minDayTime)
                    .le(SysUser::getCreateTime, maxDayTime)
                    .eq(SysUser::getExamine,2)
                    .eq(SysUser::getDelFlag,0)
                    .count();
            if(Objects.isNull(count)){
                count = 0;
            }
            yAxis.set(Long.valueOf(count));
            yAxisData.add(yAxis.get());
        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", recentlySixMonth.toArray());
        obj.put("yAxisData", yAxisData);
        return AjaxResult.success(obj);
    }






}
