package com.ruoyi.web.controller.system.count;

import cn.hutool.core.text.StrPool;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.ObtainLastSixMonthsUtil;
import com.ruoyi.system.domain.TdPropertyInfo;
import com.ruoyi.system.domain.check.TdCheckReport;
import com.ruoyi.system.domain.check.dto.CheckReportDTO;
import com.ruoyi.system.domain.count.CheckResultCountDTO;
import com.ruoyi.system.domain.count.CheckTypeCountDTO;
import com.ruoyi.system.domain.count.PropertyManagerDTO;
import com.ruoyi.system.service.ISysDictDataService;
import com.ruoyi.system.service.ITdNotifyService;
import com.ruoyi.system.service.check.TdCheckReportService;
import com.ruoyi.web.controller.manager.CheckReportManager;
import io.swagger.annotations.ApiOperation;
import lombok.val;
import org.apache.catalina.User;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * 涉密检查统计
 */
@Controller
@RequestMapping("/system/checknum")
public class TdChecknumController extends BaseController{
    private String prefix = "system/check/checknum";

    @Resource
    private TdCheckReportService checkReportService;

    @Autowired
    private ITdNotifyService tdNotifyService;

    @Resource
    private CheckReportManager checkReportManager;

    @RequiresPermissions("system:checknum:view")
    @GetMapping()
    public String checknum() {
        return prefix + "/checknum";
    }

    /**
     * 检查报告列表
     */
    @PostMapping("/list")
    @ResponseBody
    public TableDataInfo list(CheckReportDTO tdCheckReport) {
        startPage();
        List<TdCheckReport> tdCheckReports = checkReportManager.selectTdCheckReportList(tdCheckReport);
        return getDataTable(tdCheckReports);
    }

    /**
     *
     *检查近6个月统计
     * @return
     */
    @PostMapping("/countCheckMonth")
    @ResponseBody
    public AjaxResult countCheckMonth() {
        List<String> recentlySixMonth = ObtainLastSixMonthsUtil.getRecentlySixMonth();
        List<Long> yAxisData = new ArrayList<>();
        recentlySixMonth.forEach(month -> {
            // 获取指定月份的最大日期
            String lastDayOfMonth = ObtainLastSixMonthsUtil.getLastDayOfMonth(month);
            // 获取指定天的最大时间
            Date date = Date.from(LocalDate.parse(lastDayOfMonth).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date maxDayTime = ObtainLastSixMonthsUtil.getEndOfDay(date);
            // 获取最小时间
            date = Date.from(LocalDate.parse(month.concat(StrPool.DASHED).concat("01")).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date minDayTime = ObtainLastSixMonthsUtil.getStartOfDay(date);
            AtomicReference<Long> yAxis = new AtomicReference<>(0L);
            Integer count = new LambdaQueryChainWrapper<>(checkReportService.getBaseMapper())
                    .ge(TdCheckReport::getCreateTime, minDayTime)
                    .le(TdCheckReport::getCreateTime, maxDayTime).count();
            if(Objects.isNull(count)){
                count = 0;
            }
            yAxis.set(Long.valueOf(count));
            yAxisData.add(yAxis.get());
        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", recentlySixMonth.toArray());
        obj.put("yAxisData", yAxisData);
        return AjaxResult.success(obj);
    }

    /**
     *
     *统计近6个月最高得分占比
     * @return
     */
    @PostMapping("/countCheckScoresMonth")
    @ResponseBody
    public AjaxResult countCheckScoresMonth() {
        List<String> recentlySixMonth = ObtainLastSixMonthsUtil.getRecentlySixMonth();
        List<Double> yAxisData = new ArrayList<>();
        recentlySixMonth.forEach(month -> {
            // 获取指定月份的最大日期
            String lastDayOfMonth = ObtainLastSixMonthsUtil.getLastDayOfMonth(month);
            // 获取指定天的最大时间
            Date date = Date.from(LocalDate.parse(lastDayOfMonth).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date maxDayTime = ObtainLastSixMonthsUtil.getEndOfDay(date);
            // 获取最小时间
            date = Date.from(LocalDate.parse(month.concat(StrPool.DASHED).concat("01")).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date minDayTime = ObtainLastSixMonthsUtil.getStartOfDay(date);
            AtomicReference<Double> yAxis = new AtomicReference<>();
            PropertyManagerDTO tdPropertWx = new PropertyManagerDTO();
            tdPropertWx.setMinDayTime(minDayTime);
            tdPropertWx.setMaxDayTime(maxDayTime);
            Double maxCount = checkReportService.countSecretsWxMonth(tdPropertWx);
            if(Objects.isNull(maxCount)){
                yAxis.set(0.0);
            }else{
                yAxis.set(maxCount);
            }
            yAxisData.add(yAxis.get());
        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", recentlySixMonth.toArray());
        obj.put("yAxisData", yAxisData);
        return AjaxResult.success(obj);
    }










}
