package com.ruoyi.web.controller.system.count;

import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicReference;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.text.StrPool;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.ruoyi.common.utils.ObtainLastSixMonthsUtil;
import com.ruoyi.system.domain.check.TdCheckReport;
import com.ruoyi.system.domain.userbook.dto.UserBookDTO;
import com.ruoyi.system.domain.userexam.dto.request.UserExamReqDTO;
import com.ruoyi.system.domain.userexam.dto.response.UserExamExportDTO;
import com.ruoyi.system.domain.userexam.dto.response.UserExamRespDTO;
import com.ruoyi.web.controller.manager.UserExamManager;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.TdExamnum;
import com.ruoyi.system.service.ITdExamnumService;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

/**
 * 考试结果Controller
 *
 * @author ruoyi
 * @date 2024-04-10
 */
@Controller
@RequestMapping("/system/examnum")
public class TdExamnumController extends BaseController {
    private String prefix = "system/examnum";

    @Autowired
    private ITdExamnumService tdExamnumService;

    @Resource
    private UserExamManager userExamManager;


    @RequiresPermissions("system:examnum:countExam")
    @GetMapping()
    public String countExam() {
        return prefix + "/countExam";
    }


    @GetMapping("/examnum/{examId}")
    public String examnum(@PathVariable("examId") String examId, ModelMap mmap) {
        mmap.put("examId",examId);
        return prefix + "/examnum";
    }

    /**
     * 查询考试结果列表
     */
    @RequiresPermissions("system:examnum:list")
    @PostMapping("/list")
    @ResponseBody
    public TableDataInfo list(UserExamReqDTO reqDTO) {
        startPage();
        List<UserExamRespDTO> list = userExamManager.selectExamList(reqDTO);
        return getDataTable(list);
    }

    /**
     * 导出考试结果列表
     */
    @RequiresPermissions("system:examnum:export")
    @Log(title = "考试详情", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    @ResponseBody
    public AjaxResult export(UserExamReqDTO reqDTO) {
        List<UserExamRespDTO> list = userExamManager.selectExamList(reqDTO);
        List<UserExamExportDTO> list1 = Convert.toList(UserExamExportDTO.class, list);
        ExcelUtil<UserExamExportDTO> util = new ExcelUtil<UserExamExportDTO>(UserExamExportDTO.class);
        return util.exportExcel(list1, "考试数据");
    }


    @ApiOperation("查找考试总数")
    @PostMapping("/countList")
    @ResponseBody
    public AjaxResult countList() {
        return AjaxResult.success(userExamManager.selectCountList());
    }


    @GetMapping("/examList")
    public String examList(ModelMap mmap) {
        return prefix + "/examList";
    }

    @ApiOperation("最新考试列表")
    @PostMapping("/countExamList")
    @ResponseBody
    public AjaxResult countExamList() {
        return AjaxResult.success(userExamManager.selectCountExam());
    }

    /**
     *
     *近6个月考试通过统计
     * @return
     */
    @PostMapping("/countExamMonth")
    @ResponseBody
    public AjaxResult countExamMonth() {
        List<String> recentlySixMonth = ObtainLastSixMonthsUtil.getRecentlySixMonth();
        List<Long> yAxisData = new ArrayList<>();
        recentlySixMonth.forEach(month -> {
            // 获取指定月份的最大日期
            String lastDayOfMonth = ObtainLastSixMonthsUtil.getLastDayOfMonth(month);
            // 获取指定天的最大时间
            Date date = Date.from(LocalDate.parse(lastDayOfMonth).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date maxDayTime = ObtainLastSixMonthsUtil.getEndOfDay(date);
            // 获取最小时间
            date = Date.from(LocalDate.parse(month.concat(StrPool.DASHED).concat("01")).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date minDayTime = ObtainLastSixMonthsUtil.getStartOfDay(date);
            AtomicReference<Long> yAxis = new AtomicReference<>(0L);
            Integer count = userExamManager.countExamMonth(minDayTime, maxDayTime,true);
            if(Objects.isNull(count)){
                count = 0;
            }
            yAxis.set(Long.valueOf(count));
            yAxisData.add(yAxis.get());
        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", recentlySixMonth.toArray());
        obj.put("yAxisData", yAxisData);
        return AjaxResult.success(obj);
    }

    /**
     *
     *近6个月考试未通过统计
     * @return
     */
    @PostMapping("/countExamNoPass")
    @ResponseBody
    public AjaxResult countExamNoPass() {
        List<String> recentlySixMonth = ObtainLastSixMonthsUtil.getRecentlySixMonth();
        List<Long> yAxisData = new ArrayList<>();
        recentlySixMonth.forEach(month -> {
            // 获取指定月份的最大日期
            String lastDayOfMonth = ObtainLastSixMonthsUtil.getLastDayOfMonth(month);
            // 获取指定天的最大时间
            Date date = Date.from(LocalDate.parse(lastDayOfMonth).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date maxDayTime = ObtainLastSixMonthsUtil.getEndOfDay(date);
            // 获取最小时间
            date = Date.from(LocalDate.parse(month.concat(StrPool.DASHED).concat("01")).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
            Date minDayTime = ObtainLastSixMonthsUtil.getStartOfDay(date);
            AtomicReference<Long> yAxis = new AtomicReference<>(0L);
            Integer count = userExamManager.countExamMonth(minDayTime, maxDayTime,false);
            if(Objects.isNull(count)){
                count = 0;
            }
            yAxis.set(Long.valueOf(count));
            yAxisData.add(yAxis.get());
        });
        JSONObject obj = new JSONObject();
        obj.put("xAxisData", recentlySixMonth.toArray());
        obj.put("yAxisData", yAxisData);
        return AjaxResult.success(obj);
    }


}
