package com.ruoyi.common.utils;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.time.temporal.ChronoField;
import java.util.*;


/**
 * @author wangxy
 * @ClassName ObtainLastSixMonthsUtil
 * @description: 获取最近的六个月
 * @date 2023年10月11日
 * @version: 1.0.0
 */
public class ObtainLastSixMonthsUtil {

    /**
     * 获取最近十二个月的yyyy-MM集合
     *
     * @return List<String>
     */
    public static List<String> getRecentlyTwelveMonth() {
        List<String> previousMonths = new ArrayList<>();
        String currentMonth = DateUtil.format(LocalDateTime.now(), "yyyy-MM");
        previousMonths.add(currentMonth);
        for (int i = 1; i <= 11; i++) {
            DateTime dateTime = DateUtil.offsetMonth(new Date(), -i);
            String dateMonth = DateUtil.format(dateTime, "yyyy-MM");
            previousMonths.add(dateMonth);
        }
        Collections.sort(previousMonths);
        return previousMonths;
    }

    /**
     * 获取最近六个月的yyyy-MM集合
     *
     * @return List<String>
     */
    public static List<String> getRecentlySixMonth2() {
        List<String> previousMonths = new ArrayList<>();
        String currentMonth = DateUtil.format(LocalDateTime.now(), "yyyy-MM");
        previousMonths.add(currentMonth);
        for (int i = 1; i <= 5; i++) {
            DateTime dateTime = DateUtil.offsetMonth(new Date(), -i);
            String dateMonth = DateUtil.format(dateTime, "yyyy-MM");
            previousMonths.add(dateMonth);
        }
        Collections.sort(previousMonths);
        return previousMonths;
    }

    /**
     * 获取指定月份的最后一天
     *
     * @param yearMonth 格式yyyy-MM
     * @return String
     */
    public static String getLastDayOfMonth(String yearMonth) {
        DateTimeFormatter formatter = new DateTimeFormatterBuilder()
                .appendPattern("yyyy-MM[dd]")
                .parseDefaulting(ChronoField.DAY_OF_MONTH, 1)
                .toFormatter();
        LocalDate localDate = LocalDate.parse(yearMonth, formatter);
        Date date = Date.from(localDate.atStartOfDay(ZoneId.systemDefault()).toInstant());
        DateTime dateTime = DateUtil.endOfMonth(date);
        return DateUtil.format(dateTime, "yyyy-MM-dd");
    }

    /**
     * @param date
     * @return
     * @description: 获得当天最小时间
     * @author: Jeff
     * @date: 2019年12月21日
     */
    public static Date getStartOfDay(Date date) {
        return DateUtil.beginOfDay(date);
    }

    /**
     * @param date
     * @return
     * @description: 获得当天最大时间
     * @author: Jeff
     * @date: 2019年12月21日
     */
    public static Date getEndOfDay(Date date) {
        return DateUtil.endOfDay(date);
    }


    /**
     * 计算两个时间差
     */
    public static Long getDatePoor(Date endDate, Date nowDate) {
        // 获得两个时间的毫秒时间差异
        return DateUtil.between(nowDate, endDate, DateUnit.HOUR);
    }

    /**
     * 取近本年12个月的日期
     * @return
     */
    public static List<String> getRecentlySixMonth() {
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        List<String> list = new ArrayList<>();
        for (int i = 1; i < 13; i++) {
            String month = i + "";
            if (i < 10) {
                month = "0" + i;
            }
            list.add(year + "-" + month);
        }
        return list;
    }


    public static void main(String[] args) {
        List<String> recentlySixMonth = getRecentlySixMonth();
        recentlySixMonth.forEach(s -> System.out.println(getLastDayOfMonth(s)));

       /* List<String> yearMonth = getYearMonth();
        for (int i = 0; i < yearMonth.size(); i++) {
            System.out.println(yearMonth.get(i));
        }*/
    }






}
