package com.ruoyi.system.service.impl;

import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.system.domain.TdPropertyInfo;
import com.ruoyi.system.domain.count.PropertyManagerDTO;
import com.ruoyi.system.mapper.TdPropertyInfoMapper;
import com.ruoyi.system.service.ITdPropertyInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.TdPropertyManagerMapper;
import com.ruoyi.system.domain.TdPropertyManager;
import com.ruoyi.system.service.ITdPropertyManagerService;
import com.ruoyi.common.core.text.Convert;

/**
 * 密品登记Service业务层处理
 * 
 * @author ruoyi
 * @date 2024-04-26
 */
@Service
public class TdPropertyManagerServiceImpl extends ServiceImpl<TdPropertyManagerMapper, TdPropertyManager> implements ITdPropertyManagerService
{
    @Autowired
    private TdPropertyManagerMapper tdPropertyManagerMapper;

    /**
     * 查询密品登记
     * 
     * @param useId 密品登记主键
     * @return 密品登记
     */
    @Override
    public TdPropertyManager selectTdPropertyManagerByUseId(String useId)
    {
        return tdPropertyManagerMapper.selectTdPropertyManagerByUseId(useId);
    }

    /**
     * 查询密品登记列表
     * 
     * @param tdPropertyManager 密品登记
     * @return 密品登记
     */
    @DataScope(deptAlias = "d")
    @Override
    public List<TdPropertyManager> selectTdPropertyManagerList(TdPropertyManager tdPropertyManager)
    {
        return tdPropertyManagerMapper.selectTdPropertyManagerList(tdPropertyManager);
    }

    /**
     * 新增密品登记
     * 
     * @param tdPropertyManager 密品登记
     * @return 结果
     */
    @Override
    public int insertTdPropertyManager(TdPropertyManager tdPropertyManager)
    {
        return tdPropertyManagerMapper.insertTdPropertyManager(tdPropertyManager);
    }

    /**
     * 修改密品登记
     * 
     * @param tdPropertyManager 密品登记
     * @return 结果
     */
    @Override
    public int updateTdPropertyManager(TdPropertyManager tdPropertyManager)
    {
        return tdPropertyManagerMapper.updateTdPropertyManager(tdPropertyManager);
    }

    /**
     * 批量删除密品登记
     * 
     * @param useIds 需要删除的密品登记主键
     * @return 结果
     */
    @Override
    public int deleteTdPropertyManagerByUseIds(String useIds)
    {
        return tdPropertyManagerMapper.deleteTdPropertyManagerByUseIds(Convert.toStrArray(useIds));
    }

    /**
     * 删除密品登记信息
     * 
     * @param useId 密品登记主键
     * @return 结果
     */
    @Override
    public int deleteTdPropertyManagerByUseId(String useId)
    {
        return tdPropertyManagerMapper.deleteTdPropertyManagerByUseId(useId);
    }

    @Override
    public Integer countMessageMonth(PropertyManagerDTO propertyManagerDTO) {
        return tdPropertyManagerMapper.countMessageMonth(propertyManagerDTO);
    }

    @Override
    public Integer countSecretsMonth(PropertyManagerDTO propertyManagerDTO) {
        return tdPropertyManagerMapper.countSecretsMonth(propertyManagerDTO);
    }

    @Override
    public Integer countDestoryMonth(PropertyManagerDTO propertyManagerDTO) {
        return tdPropertyManagerMapper.countDestoryMonth(propertyManagerDTO);
    }

    @Override
    public Integer countMessageWxMonth(PropertyManagerDTO propertyManagerDTO) {
        return tdPropertyManagerMapper.countMessageWxMonth(propertyManagerDTO);
    }


    @Override
    public Integer countSecretsDestoryMonth(PropertyManagerDTO propertyManagerDTO) {
        return tdPropertyManagerMapper.countSecretsDestoryMonth(propertyManagerDTO);
    }

    @Override
    public Integer countSecretsWxMonth(PropertyManagerDTO propertyManagerDTO) {
        return tdPropertyManagerMapper.countSecretsWxMonth(propertyManagerDTO);
    }
}
